

#include "allegro5/allegro.h"
#include "allegro5/allegro_image.h"
#include "allegro5/allegro_font.h"
#include "allegro5/allegro_ttf.h"
#include "allegro5/allegro_primitives.h"
#include <cstdio>

ALLEGRO_DISPLAY* display = NULL;
ALLEGRO_EVENT_QUEUE* queue = NULL;
ALLEGRO_BITMAP* image = NULL;
ALLEGRO_TIMER* timer = NULL;
ALLEGRO_FONT* font1 = NULL;
ALLEGRO_FONT* font2 = NULL;
ALLEGRO_COLOR color1;
ALLEGRO_COLOR color2;


int keys[ALLEGRO_KEY_MAX];

ALLEGRO_BITMAP* MakeTile() {
   ALLEGRO_BITMAP* tile = al_create_bitmap(256,256);
   al_set_target_bitmap(tile);
   for (int y = 0 ; y < 256 ; ++y) {
      for (int x = 0 ; x < 256 ; ++x) {
         al_draw_pixel(x , y , al_map_rgba(x , y , 0 , 255));
      }
   }
   return tile;
}

class CONVERSATION {
public :
   const char* text;
   ALLEGRO_COLOR text_color;
   CONVERSATION* next;
   CONVERSATION* prev;
   ALLEGRO_FONT* font;
};
CONVERSATION* CreateConversationElement(const char* text , ALLEGRO_COLOR text_color , ALLEGRO_FONT* font) {
   CONVERSATION* c = new CONVERSATION();
   c->text = text;
   c->text_color = text_color;
   c->font = font;
   return c;
}

CONVERSATION* SetConversationPrev(CONVERSATION* c , CONVERSATION* cprev) {
   c->prev = cprev;
   if (cprev) {
      cprev->next = c;
   }
   return cprev;
}


CONVERSATION* SetConversationNext(CONVERSATION* c , CONVERSATION* cnext) {
   c->next = cnext;
   if (cnext) {
      cnext->prev = c;
   }
   return cnext;
}

CONVERSATION* MakeFirstConversation() {
   CONVERSATION* npcAroot = CreateConversationElement("Hello. This is the beginning of the conversation." , color1 , font1);
   CONVERSATION* npcBroot = CreateConversationElement("Oh? We're going to have a conversation?" , color2 , font2);
   SetConversationPrev(npcAroot , 0);
   SetConversationPrev(npcBroot , npcAroot);

   CONVERSATION* A2 = CreateConversationElement("Yes. Is there anything you'd like to talk about?" , color1 , font1);
   SetConversationPrev(A2 , npcBroot);
   
   CONVERSATION* B2 = CreateConversationElement("Could we talk about NPC programming?" , color2 , font2);
   SetConversationPrev(B2 , A2);
   
   CONVERSATION* A3 = CreateConversationElement("Sure, what do you want to know?" , color1 , font1);
   SetConversationPrev(A3 , B2);
   
   CONVERSATION* B3 = CreateConversationElement("How do I make a conversation?" , color2 , font2);
   SetConversationPrev(B3 , A3);
   
   CONVERSATION* A4 = CreateConversationElement("Well, you could use a linked list." , color1 , font1);
   SetConversationPrev(A4 , B3);
   SetConversationNext(A4 , 0);
   
   return npcAroot;
}

void DestroyConversation(CONVERSATION* root) {
   if (root->next) {
      DestroyConversation(root->next);
   }
   delete root;
}

int main(int argc , char** argv) {
   
   for (int i = 0 ; i < ALLEGRO_KEY_MAX ; ++i) {keys[i] = false;}
   
   if (!al_init()) {return -1;}
   if (!al_init_image_addon()) {return -2;}
   if (!al_install_keyboard()) {return -3;}
   if (!al_init_font_addon()) {return -4;}
   if (!al_init_ttf_addon()) {return -5;}
   if (!al_init_primitives_addon()) {return -6;}
   
   
   al_set_new_display_flags(ALLEGRO_WINDOWED | ALLEGRO_RESIZABLE);
   
   display = al_create_display(640,480);
   
   queue = al_create_event_queue();
   if (!queue) {return -4;}
   
   timer = al_create_timer(1.0/60.0);
   if (!timer) {return -5;}
   
   al_register_event_source(queue , al_get_display_event_source(display));
   al_register_event_source(queue , al_get_keyboard_event_source());
   al_register_event_source(queue , al_get_timer_event_source(timer));
   
   
   image = MakeTile();
   al_set_target_bitmap(al_get_backbuffer(display));
   
   font1 = al_load_ttf_font("consola.ttf" , 20 , 0);
   font2 = al_load_ttf_font("verdana.ttf" , 20 , 0);
   
   color1 = al_map_rgb(255,255,0);
   color2 = al_map_rgb(255,127,64);
   
   
   
   bool redraw = true;
   bool quit = false;
   
   CONVERSATION* c = MakeFirstConversation();
   
   do {
      if (redraw) {
         al_clear_to_color(al_map_rgb(255,255,255));
/*
         for (int y = 0 ; y < al_get_display_height(display) / al_get_bitmap_height(image) + 1 ; ++y) {
            for (int x = 0 ; x < al_get_display_width(display) / al_get_bitmap_width(image) + 1; ++x) {
               al_draw_bitmap(image , x*al_get_bitmap_width(image) , y*al_get_bitmap_height(image) , 0);
            }
         }
*/         
         // draw current part of conversation
         int tw = al_get_text_width(c->font , c->text);
         int th = al_get_font_line_height(c->font);
         int tx = al_get_display_width(display)/2 - tw/2 - 20;
         int ty = al_get_display_height(display)/2 - th/2 - 20;
         
         al_draw_filled_rectangle(tx , ty , tx + tw + 40 , ty + th + 40 , al_map_rgb(0,0,0));
         al_draw_text(c->font , c->text_color , al_get_display_width(display)/2 , al_get_display_height(display)/2 ,
                      ALLEGRO_ALIGN_CENTER , c->text);
         
         
         al_flip_display();
         redraw = false;
      }
      do {
         ALLEGRO_EVENT ev;
         al_wait_for_event(queue , &ev);
         if (ev.type == ALLEGRO_EVENT_DISPLAY_CLOSE) {quit = true;}
         if (ev.type == ALLEGRO_EVENT_KEY_DOWN && ev.keyboard.keycode == ALLEGRO_KEY_ESCAPE) {quit = true;}
         if (ev.type == ALLEGRO_EVENT_DISPLAY_RESIZE) {
            al_acknowledge_resize(display);
            redraw = true;
         }
         if (ev.type == ALLEGRO_EVENT_TIMER) {redraw = true;}
         
         if (ev.type == ALLEGRO_EVENT_KEY_DOWN || ev.type == ALLEGRO_EVENT_KEY_UP) {
            if (ev.type == ALLEGRO_EVENT_KEY_DOWN) {keys[ev.keyboard.keycode] = true;}
            if (ev.type == ALLEGRO_EVENT_KEY_UP) {keys[ev.keyboard.keycode] = false;}
         }
         if (ev.type == ALLEGRO_EVENT_KEY_DOWN && (ev.keyboard.keycode == ALLEGRO_KEY_ENTER || ev.keyboard.keycode == ALLEGRO_KEY_SPACE)) {
            if (c->next) {
               c = c->next;
            }
            redraw = true;
         }
         if (ev.type == ALLEGRO_EVENT_KEY_DOWN && ev.keyboard.keycode == ALLEGRO_KEY_BACKSPACE) {
            if (c->prev) {
               c = c->prev;
            }
            redraw = true;
         }
         
         
      } while (!al_is_event_queue_empty(queue));
         
   } while (!quit);
   
   DestroyConversation(c);
   
   return 0;
}

